///////////////////////////////////////////////////////////////////////////////
// File: SocketServer.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "SocketServer.h"

#include "JetByteTools/CoreTools/FileUtils.h"
#include "JetByteTools/CoreTools/StringUtils.h"
#include "JetByteTools/CoreTools/DebugTrace.h"
#include "JetByteTools/CoreTools/Exception.h"

#include "JetByteTools/SocketTools/IDatagramServerSocket.h"

#pragma hdrstop

#include "JetByteTools/SocketTools/AddressRenderer.h"

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::Output;
using JetByteTools::Core::DumpData;
using JetByteTools::Core::CException;

using JetByteTools::IO::IAllocateBuffers;
using JetByteTools::IO::CSmartBuffer;
using JetByteTools::IO::IIOPool;

using JetByteTools::Socket::CDatagramSocketServer;
using JetByteTools::Socket::IDatagramServerSocket;
using JetByteTools::Socket::IAllocateDatagramServerSockets;
using JetByteTools::Socket::IFullAddress;
using JetByteTools::Socket::ILimitConnections;
using JetByteTools::Socket::CAddressRenderer;
using JetByteTools::Socket::ListenBacklog;

///////////////////////////////////////////////////////////////////////////////
// CSocketServer
///////////////////////////////////////////////////////////////////////////////

CSocketServer::CSocketServer(
   const IFullAddress &address,
   const ListenBacklog listenBacklog,
   IIOPool &pool,
   IAllocateDatagramServerSockets &socketAllocator,
   IAllocateBuffers &bufferAllocator,
   ILimitConnections &connectionLimiter)
   :  CDatagramSocketServer(address, listenBacklog, *this, pool, socketAllocator, bufferAllocator, connectionLimiter)
{

}

CSocketServer::~CSocketServer()
{
   WaitForShutdownToComplete();
}

void CSocketServer::OnReadCompleted(
   IDatagramServerSocket &socket,
   CSmartBuffer &buffer)
{
   try
   {
      EchoMessage(socket, buffer);
   }
   catch(const CException &e)
   {
      Output(_T("ReadCompleted - Exception - ") + e.GetDetails());
   }
   catch(...)
   {
      Output(_T("ReadCompleted - Unexpected exception"));
   }
}

void CSocketServer::EchoMessage(
   IDatagramServerSocket &socket,
   CSmartBuffer &buffer) const
{
   DEBUG_ONLY(Output(_T(" Local Address: ") + CAddressRenderer::AsString(socket.GetLocalAddress(), true)));
   DEBUG_ONLY(Output(_T("Remote Address: ") + CAddressRenderer::AsString(socket.GetRemoteAddress(), true)));

   DEBUG_ONLY(Output(_T("Data Echoed -\r\n") + DumpData(buffer->GetMemory(), buffer->GetUsed(), 60, true)));

   socket.Write(buffer);
}

///////////////////////////////////////////////////////////////////////////////
// End of file: SocketServer.cpp
///////////////////////////////////////////////////////////////////////////////
