///////////////////////////////////////////////////////////////////////////////
// File: AsyncSocketConnectionManager.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "AsyncSocketConnectionManager.h"

#include "JetByteTools/CoreTools/Exception.h"
#include "JetByteTools/CoreTools/FileUtils.h"
#include "JetByteTools/CoreTools/StringUtils.h"
#include "JetByteTools/CoreTools/DebugTrace.h"

#include "JetByteTools/SocketTools/IDatagramSocket.h"
#include "JetByteTools/SocketTools/IAllocateDatagramSockets.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::OutputEx;
using JetByteTools::Core::CException;
using JetByteTools::Core::ToString;

using JetByteTools::IO::IIOPool;
using JetByteTools::IO::CSmartBuffer;
using JetByteTools::IO::IAllocateBuffers;

using JetByteTools::Socket::IDatagramSocket;
using JetByteTools::Socket::IAllocateDatagramSockets;

///////////////////////////////////////////////////////////////////////////////
// CAsyncSocketConnectionManager
///////////////////////////////////////////////////////////////////////////////

CAsyncSocketConnectionManager::CAsyncSocketConnectionManager(
   IIOPool &pool,
   IAllocateDatagramSockets &socketAllocator,
   IAllocateBuffers &bufferAllocator)
   :  JetByteTools::Socket::CDatagramSocketConnectionManager(*this, pool, socketAllocator, bufferAllocator)
{

}

void CAsyncSocketConnectionManager::OnReadCompleted(
   IDatagramSocket &socket,
   CSmartBuffer &buffer)
{
   try
   {
      OutputEx(_T("ReadCompleted: ") + ToString(buffer->GetUsed()) + _T(" bytes"));

      socket.Read();
   }
   catch(const CException &e)
   {
      OutputEx(_T("ReadCompleted - Exception - ") + e.GetDetails());
      socket.Shutdown();
   }
   catch(...)
   {
      OutputEx(_T("ReadCompleted - Unexpected exception"));
      socket.Shutdown();
   }
}

///////////////////////////////////////////////////////////////////////////////
// End of file: AsyncSocketConnectionManager.cpp
///////////////////////////////////////////////////////////////////////////////
