#pragma once
#ifndef ECHO_SERVER_UDP_TEST_DATAGRAM_SOCKET_CONNECTION_MANAGER_INCLUDED__
#define ECHO_SERVER_UDP_TEST_DATAGRAM_SOCKET_CONNECTION_MANAGER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: DatagramSocketConnectionManager.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2006 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SocketTools/DatagramSocketConnectionManager.h"
#include "JetByteTools/SocketTools/ReadTimeoutDatagramSocketConnectionFilter.h"

#include "ServerCommon/DatagramSocketConnectionManagerCallback.h"
#include "ServerCommon/ThreadedCallbackTimerQueue.h"

#include "JetByteTools/Win32Tools/CriticalSection.h"

#include <map>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

class CTestDatagrams;
class ITestDatagram;

///////////////////////////////////////////////////////////////////////////////
// CDatagramSocketConnectionManager
///////////////////////////////////////////////////////////////////////////////

class CDatagramSocketConnectionManager :
   public JetByteTools::Socket::CFilteringDatagramSocketConnectionManager,
   private CDatagramSocketConnectionManagerCallback,
   private JetByteTools::Socket::CReadTimeoutDatagramSocketConnectionFilter::TimerCallback
{
   public :

      CDatagramSocketConnectionManager(
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateDatagramSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         CTestDatagrams &datagrams,
         JetByteTools::Milliseconds readTimeout);

      CDatagramSocketConnectionManager(
         const CDatagramSocketConnectionManager &rhs) = delete;

      ~CDatagramSocketConnectionManager();

      CDatagramSocketConnectionManager &operator=(
         const CDatagramSocketConnectionManager &rhs) = delete;

      void WaitForShutdownToComplete() override;

      bool WaitForShutdownToComplete(
         JetByteTools::Milliseconds timeout) override;

      bool DumpErrors() const;

   private :

      void OnRecvFromCompleted(
         JetByteTools::Socket::IDatagramSocket &socket,
         const JetByteTools::Socket::IAddress &address,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void OnSendToCompleted(
         JetByteTools::Socket::IDatagramSocket &socket,
         const JetByteTools::Socket::IAddress &address,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void OnSendToCompletionError(
         JetByteTools::Socket::IDatagramSocket &socket,
         const JetByteTools::Socket::IAddress &address,
         JetByteTools::IO::CSmartBuffer &buffer,
         DWORD lastError) override;

      void OnError(
         const JetByteTools::Core::_tstring &message) override;

      void OnError(
         const JetByteTools::Core::_tstring &message,
         DWORD lastError) override;

      void OnConnectionClosure(
         JetByteTools::Socket::IDatagramSocket &socket,
         JetByteTools::Socket::ConnectionClosureReason reason) override;

      void OnConnectionClosed(
         JetByteTools::Socket::IDatagramSocket &socket) override;

      // Implement CReadTimeoutDatagramSocketConnectionFilter::Timer

      void OnTimer(
         JetByteTools::Socket::IDatagramSocket &socket,
         JetByteTools::Socket::CReadTimeoutDatagramSocketConnectionFilter::UserData userData) override;

      JetByteTools::Core::CLockableObject m_lock;

      CThreadedCallbackTimerQueue m_timerQueue;

      JetByteTools::Socket::CReadTimeoutDatagramSocketConnectionFilter m_readTimeoutFilter;

      const JetByteTools::Milliseconds m_readTimeout;

      typedef std::map<JetByteTools::Core::_tstring, size_t> StringErrors;

      StringErrors m_errors;

      CTestDatagrams &m_datagrams;
};

#endif // ECHO_SERVER_UDP_TEST_DATAGRAM_SOCKET_CONNECTION_MANAGER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: DatagramSocketConnectionManager.h
///////////////////////////////////////////////////////////////////////////////
