#pragma once
#ifndef ECHO_SERVER_UDP_TEST_TEST_DATAGRAMS_INCLUDED__
#define ECHO_SERVER_UDP_TEST_TEST_DATAGRAMS_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: TestDatagrams.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2009 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/WinsockWrapper.h"

#include "JetByteTools/CoreTools/Types.h"
#include "JetByteTools/CoreTools/tstring.h"
#include "JetByteTools/CoreTools/ManualResetEvent.h"

#include <wtypes.h>

#include <vector>
#include <map>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace Socket
   {
      class IDatagramSocket;
   }
}

class ITestDatagram;

///////////////////////////////////////////////////////////////////////////////
// CTestDatagrams
///////////////////////////////////////////////////////////////////////////////

class CTestDatagrams
{
   public :

      typedef DWORD DataLength;

      explicit CTestDatagrams(
         DWORD repeatCount);

      CTestDatagrams(
         const CTestDatagrams &rhs) = delete;

      ~CTestDatagrams();

      CTestDatagrams &operator=(
         const CTestDatagrams &rhs) = delete;

      HANDLE GetWaitHandle() const;

      void SetRepeatCount(
         size_t repeats);

      void AddDatagram(
         ITestDatagram *pDatagram);

      ITestDatagram &GetDatagram(
         DWORD id) const;

      void ValidateReceivedData(
         const BYTE *pData,
         DataLength dataLength);

      DWORD GetNumberOfRepliesPending() const;

      void Fail();

      bool HasFailed() const;

      bool DataflowComplete() const;

      bool DataflowSucceeded() const;

      size_t GetNumSent() const;

      size_t GetNumReceived() const;

      DWORD GetAverageRTT() const;

      DWORD GetMaxRTT() const;

      DWORD GetMinRTT() const;

      void DumpRTT() const;

   private :

      const DWORD m_repeatCount;

      volatile bool m_succeeded;

      typedef std::vector<ITestDatagram *> Datagrams;

      Datagrams m_datagrams;

      JetByteTools::Core::CManualResetEvent m_completeEvent;

      volatile long m_repliesPending;
};

#endif // ECHO_SERVER_UDP_TEST_TEST_DATAGRAMS_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: TestDatagrams.h
///////////////////////////////////////////////////////////////////////////////
