///////////////////////////////////////////////////////////////////////////////
// File: StreamSocketServerExCallback.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"
#include "JetByteTools/Admin/WinsockWrapper.h"

#include "StreamSocketServerExCallback.h"

#include "JetByteTools/CoreTools/DebugTrace.h"
#include "JetByteTools/CoreTools/ToString.h"
#include "JetByteTools/CoreTools/ErrorCodeToErrorMessage.h"

#include "JetByteTools/SocketTools/StreamSocketServerExCallback.h"
#include "JetByteTools/SocketTools/IStreamSocket.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::_tstring;
using JetByteTools::Core::ToString;
using JetByteTools::Core::IIndexedOpaqueUserData;
using JetByteTools::Core::Output;
using JetByteTools::Core::OutputEx;

using JetByteTools::Core::ErrorCodeToErrorMessage;

using JetByteTools::IO::IBuffer;
using JetByteTools::IO::CSmartBuffer;

using JetByteTools::Socket::IStreamSocket;
using JetByteTools::Socket::IAddress;
using JetByteTools::Socket::ShutdownHow;
using JetByteTools::Socket::ConnectionClosureReason;
using JetByteTools::Socket::GetClosureReasonAsString;

///////////////////////////////////////////////////////////////////////////////
// CStreamSocketServerExCallback
///////////////////////////////////////////////////////////////////////////////

void CStreamSocketServerExCallback::OnStartAcceptingConnections()
{
   Output(_T("OnStartAcceptingConnections"));
}

void CStreamSocketServerExCallback::OnStopAcceptingConnections()
{
   Output(_T("OnStopAcceptingConnections"));
}

void CStreamSocketServerExCallback::OnAcceptError(
   const DWORD lastError)
{
   OnError(_T("CStreamSocketServerExCallback::OnAcceptError"), lastError);
}

void CStreamSocketServerExCallback::OnIncrementPendingReceives(
   const DWORD /*pendingReceives*/)
{
}

void CStreamSocketServerExCallback::OnDecrementPendingReceives(
   const DWORD /*pendingReceives*/)
{
}

void CStreamSocketServerExCallback::OnSocketCreated(
   SOCKET /*s*/)
{

}

void CStreamSocketServerExCallback::OnPreOutgoingConnect(
   IIndexedOpaqueUserData & /*userData*/,
   const IAddress & /*address*/,
   const void * /*pUserData*/)
{
}

void CStreamSocketServerExCallback::OnConnectionEstablished(
   IStreamSocket & /*socket*/,
   const IAddress & /*address*/)
{
}

void CStreamSocketServerExCallback::OnOutgoingConnectionFailed(
   IStreamSocket & /*socket*/,
   const IAddress & /*address*/,
   const DWORD /*lastError*/)
{
}

void CStreamSocketServerExCallback::OnConnectionClientClose(
    IStreamSocket & /*socket*/)
{
   Output(_T("OnConnectionClientClose"));
}

void CStreamSocketServerExCallback::OnConnectionReset(
    IStreamSocket & /*socket*/,
    DWORD lastError)
{
   Output(_T("OnConnectionReset: ") + ErrorCodeToErrorMessage(lastError));
}

void CStreamSocketServerExCallback::OnConnectionClosure(
   IStreamSocket & /*socket*/,
   const ConnectionClosureReason reason)
{
   Output(_T("OnConnectionClosure: ") + GetClosureReasonAsString(reason));
}

void CStreamSocketServerExCallback::OnConnectionClosed(
   IStreamSocket & /*socket*/)
{
   Output(_T("OnConnectionClosed"));
}

void CStreamSocketServerExCallback::OnSocketReleased(
   IIndexedOpaqueUserData & /*userData*/)
{
}

void CStreamSocketServerExCallback::OnError(
   const _tstring &message)
{
   OutputEx(_T("CStreamSocketServerExCallback::OnError: ") + message);
}

void CStreamSocketServerExCallback::OnError(
   const _tstring &message,
   const DWORD lastError)
{
   OutputEx(_T("CStreamSocketServerExCallback::OnError: ") + message + _T(": ") + ErrorCodeToErrorMessage(lastError));
}

void CStreamSocketServerExCallback::OnReadCompleted(
   IStreamSocket & /*socket*/,
   CSmartBuffer & /*buffer*/)
{
}

void CStreamSocketServerExCallback::OnReadOOBCompleted(
   IStreamSocket & /*socket*/,
   CSmartBuffer & /*buffer*/)
{
}

void CStreamSocketServerExCallback::OnReadCompletionError(
   IStreamSocket & /*socket*/,
   CSmartBuffer & /*buffer*/,
   const DWORD lastError)
{
   OnError(_T("CStreamSocketServerExCallback::OnReadCompletionError"), lastError);
}

void CStreamSocketServerExCallback::OnWriteCompleted(
   IStreamSocket & /*socket*/,
   CSmartBuffer &buffer)
{
   // The check below has never failed in production code
   // but can do so if you run out of non-paged pool at the right point...
   // If you only have a single write pending you can resync and resend based
   // on what you actually managed to send...

   const DWORD expected = IBuffer::GetTotalLength(buffer->GetWSABUF(), buffer->GetWSABUFCount());

   if (buffer->GetUsed() != expected)
   {
      OnError(_T("CStreamSocketServerExCallback::OnWriteCompleted - Socket write where not all data was written - expected: ") +
         ToString(expected) + _T(" sent:") +
         ToString(buffer->GetUsed()));
   }
}

void CStreamSocketServerExCallback::OnWriteCompletionError(
   IStreamSocket & /*socket*/,
   CSmartBuffer & /*buffer*/,
   const DWORD lastError)
{
   OnError(_T("CStreamSocketServerExCallback::OnWriteCompletionError"), lastError);
}

void CStreamSocketServerExCallback::OnShutdownInitiated()
{
   Output(_T("OnShutdownInitiated"));
}

void CStreamSocketServerExCallback::OnShutdownComplete()
{
   Output(_T("OnShutdownComplete"));
}

void CStreamSocketServerExCallback::OnMaximumConnections()
{
   Output(_T("OnMaximumConnections"));
}

void CStreamSocketServerExCallback::OnAllConnectionsAborted(
   const DWORD /*numberOfConnectionsAborted*/)
{
}

///////////////////////////////////////////////////////////////////////////////
// End of file: StreamSocketServerExCallback.cpp
///////////////////////////////////////////////////////////////////////////////
